package l09_threads;

import utilities.ThreadHelperMethods;

public class DualSynchronizationTest {

	static class DualSynchronization {
		private static final Object lock = new Object();

		public static synchronized void f() {
			System.out.println("Inside f()");
			ThreadHelperMethods.safeSleep(500);
			System.out.println("Leaving f()");
		}

		public static void g() {
			synchronized (lock) {
				System.out.println("Inside g()");
				ThreadHelperMethods.safeSleep(600);
				System.out.println("Leaving g()");
			}
		}
	}

	//	The two synchronizations are independent. This is demonstrated in main() by
	//	creating a Thread that calls f(). The main() thread is used to call g(). You
	//	can see from the output that both methods are running at the same time, so
	//	neither one is blocked by the synchronization of the other. 

	public static void main(String[] args) {
		new Thread() {
			public void run() { DualSynchronization.f(); }
		}.start();
		new Thread() {
			public void run() { DualSynchronization.g(); }
		}.start();
	}
}

