package l09_threads;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;

public class ProducerConsumerTest {
	public static void main(String[] args) {

		int numberProducers = (args.length >= 1) ? Integer.parseInt(args[0]) : 2;
		int bufferLength = (args.length >= 2) ? Integer.parseInt(args[1]) : 1;

		Buffer buffer = new Buffer(bufferLength);
		for (int i = 0; i < numberProducers; i++) {
			new Producer(buffer, i).start();
			new Consumer(buffer, i).start();
		}
	}

	static class Producer extends Thread {
		private final Buffer buffer;
		private final int number;

		Producer(Buffer buffer, int number) {
			this.buffer = buffer;
			this.number = number;
		}

		@Override
		public void run() {
			for (int i = 0; i < 5; i++) {
				buffer.put(RandomGenerators.getIntThread(100), number);
				ThreadHelperMethods.safeSleep(30);
			}
		}
	}

	static class Consumer extends Thread {
		private final Buffer buffer;
		private final int number;

		Consumer(Buffer buffer, int number) {
			this.buffer = buffer;
			this.number = number;
		}

		@Override
		public void run() {
			for (int i = 0; i < 5; i++) {
				buffer.get(number);
				ThreadHelperMethods.safeSleep(200);
			}
		}
	}

	//	The only place you can call wait(), notify(), or notifyAll() is within a
	//	synchronized method or block (sleep() can be called within non-synchronized
	//	methods since it doesn't manipulate the lock). 

	static class Buffer {

		private int bufferPosition = 0;
		private final int bufferLength;
		private final int[] buffer;

		public Buffer(int bufferLength) {
			this.bufferLength = bufferLength;
			this.buffer = new int[bufferLength];
		}

		synchronized int get(int num) {
			while (bufferPosition == 0) {
				safeWait();
			}
			notifyAll();
			bufferPosition--;
			System.out.println("\tConsumer #" + num + " got: " + buffer[bufferPosition]);
			return buffer[bufferPosition];
		}

		synchronized void put(int value, int num) {
			while (bufferPosition >= bufferLength) {
				safeWait();
			}
			notifyAll();
			buffer[bufferPosition] = value;
			System.out.println("Producer #" + num + " put: " + buffer[bufferPosition]);
			bufferPosition++;
		}

		private void safeWait() {
			try {
				wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}

